@@ -1,7 +1,7 @@
 /* lepdflib.cc
 
-   Copyright 2009-2010 Taco Hoekwater <taco@luatex.org>
-   Copyright 2009-2010 Hartmut Henkel <hartmut@luatex.org>
+   Copyright 2009-2011 Taco Hoekwater <taco@luatex.org>
+   Copyright 2009-2011 Hartmut Henkel <hartmut@luatex.org>
 
    This file is part of LuaTeX.
 
@@ -19,8 +19,8 @@
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
 static const char _svn_version[] =
-    "$Id: lepdflib.cc 3961 2010-11-13 11:59:05Z hhenkel $ "
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.65.0/source/texk/web2c/luatexdir/lua/lepdflib.cc $";
+    "$Id: lepdflib.cc 4134 2011-04-11 20:54:58Z hhenkel $ "
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.70.1/source/texk/web2c/luatexdir/lua/lepdflib.cc $";
 
 #include "image/epdf.h"
 
@@ -54,6 +54,7 @@ static const char *ErrorCodeNames[] = { "None", "OpenFile", "BadCatalog",
 #define M_Annots           "Annots"
 #define M_Array            "Array"
 #define M_Catalog          "Catalog"
+#define M_EmbFile          "EmbFile"
 #define M_Dict             "Dict"
 #define M_GooString        "GooString"
 #define M_LinkDest         "LinkDest"
@@ -88,6 +89,7 @@ new_poppler_userdata(AnnotBorder);
 new_poppler_userdata(Annots);
 new_poppler_userdata(Array);
 new_poppler_userdata(Catalog);
+new_poppler_userdata(EmbFile);
 new_poppler_userdata(Dict);
 //new_poppler_userdata(GooString);
 new_poppler_userdata(LinkDest);
@@ -680,6 +682,56 @@ static int m_Catalog_findDest(lua_State * L)
 }
 
 m_poppler_get_poppler(Catalog, Object, getDests);
+m_poppler_get_INT(Catalog, numEmbeddedFiles);
+
+static int m_Catalog_embeddedFile(lua_State * L)
+{
+    EmbFile *ef;
+    int i, len;
+    udstruct *uin, *uout;
+    uin = (udstruct *) luaL_checkudata(L, 1, M_Catalog);
+    if (uin->pd != NULL && uin->pd->pc != uin->pc)
+        pdfdoc_changed_error(L);
+    i = luaL_checkint(L, 2);
+    len = ((Catalog *) uin->d)->numEmbeddedFiles();
+    if (i > 0 && i <= len) {
+        ef = ((Catalog *) uin->d)->embeddedFile(i - 1);
+        if (ef != NULL) {
+            uout = new_EmbFile_userdata(L);
+            uout->d = ef;
+            uout->pc = uin->pc;
+            uout->pd = uin->pd;
+        } else
+            lua_pushnil(L);
+    } else
+        lua_pushnil(L);
+    return 1;
+}
+
+m_poppler_get_INT(Catalog, numJS);
+
+static int m_Catalog_getJS(lua_State * L)
+{
+    GooString *gs;
+    int i, len;
+    udstruct *uin;
+    uin = (udstruct *) luaL_checkudata(L, 1, M_Catalog);
+    if (uin->pd != NULL && uin->pd->pc != uin->pc)
+        pdfdoc_changed_error(L);
+    i = luaL_checkint(L, 2);
+    len = ((Catalog *) uin->d)->numJS();
+    if (i > 0 && i <= len) {
+        gs = ((Catalog *) uin->d)->getJS(i - 1);
+        if (gs != NULL)
+            lua_pushlstring(L, gs->getCString(), gs->getLength());
+        else
+            lua_pushnil(L);
+        delete gs;
+    } else
+        lua_pushnil(L);
+    return 1;
+}
+
 m_poppler_get_poppler(Catalog, Object, getOutline);
 m_poppler_get_poppler(Catalog, Object, getAcroForm);
 
@@ -696,12 +748,59 @@ static const struct luaL_Reg Catalog_m[] = {
     {"findPage", m_Catalog_findPage},
     {"findDest", m_Catalog_findDest},
     {"getDests", m_Catalog_getDests},
+    {"numEmbeddedFiles", m_Catalog_numEmbeddedFiles},
+    {"embeddedFile", m_Catalog_embeddedFile},
+    {"numJS", m_Catalog_numJS},
+    {"getJS", m_Catalog_getJS},
     {"getOutline", m_Catalog_getOutline},
     {"getAcroForm", m_Catalog_getAcroForm},
     {"__tostring", m_Catalog__tostring},
     {NULL, NULL}                // sentinel
 };
 
+//**********************************************************************
+// EmbFile
+
+m_poppler_get_GOOSTRING(EmbFile, name);
+m_poppler_get_GOOSTRING(EmbFile, description);
+m_poppler_get_INT(EmbFile, size);
+m_poppler_get_GOOSTRING(EmbFile, modDate);
+m_poppler_get_GOOSTRING(EmbFile, createDate);
+m_poppler_get_GOOSTRING(EmbFile, checksum);
+m_poppler_get_GOOSTRING(EmbFile, mimeType);
+
+static int m_EmbFile_streamObject(lua_State * L)
+{
+    udstruct *uin, *uout;
+    uin = (udstruct *) luaL_checkudata(L, 1, M_EmbFile);
+    if (uin->pd != NULL && uin->pd->pc != uin->pc)
+        pdfdoc_changed_error(L);
+    uout = new_Object_userdata(L);
+    uout->d = new Object();     // automatic init to type "none"
+    ((EmbFile *) uin->d)->streamObject().copy((Object *) uout->d);
+    uout->pc = uin->pc;
+    uout->pd = uin->pd;
+    return 1;
+}
+
+m_poppler_get_BOOL(EmbFile, isOk);
+
+m_poppler__tostring(EmbFile);
+
+static const struct luaL_Reg EmbFile_m[] = {
+    {"name", m_EmbFile_name},
+    {"description", m_EmbFile_description},
+    {"size", m_EmbFile_size},
+    {"modDate", m_EmbFile_modDate},
+    {"createDate", m_EmbFile_createDate},
+    {"checksum", m_EmbFile_checksum},
+    {"mimeType", m_EmbFile_mimeType},
+    {"streamObject", m_EmbFile_streamObject},
+    {"isOk", m_EmbFile_isOk},
+    {"__tostring", m_EmbFile__tostring},
+    {NULL, NULL}                // sentinel
+};
+
 //**********************************************************************
 // Dict
 
@@ -1026,6 +1125,29 @@ static const struct luaL_Reg Links_m[] = {
 //**********************************************************************
 // Object
 
+// Special type checking.
+#define m_Object_isType(function)                                          \
+static int m_Object_##function(lua_State * L)                              \
+{                                                                          \
+    udstruct *uin;                                                         \
+    uin = (udstruct *) luaL_checkudata(L, 1, M_Object);                    \
+    if (uin->pd != NULL && uin->pd->pc != uin->pc)                         \
+        pdfdoc_changed_error(L);                                           \
+    if (lua_gettop(L) >= 2) {                                              \
+        if (lua_isstring(L, 2)                                             \
+            && ((Object *) uin->d)->function((char *) lua_tostring(L, 2))) \
+            lua_pushboolean(L, 1);                                         \
+        else                                                               \
+            lua_pushboolean(L, 0);                                         \
+    } else {                                                               \
+        if (((Object *) uin->d)->function())                               \
+            lua_pushboolean(L, 1);                                         \
+        else                                                               \
+            lua_pushboolean(L, 0);                                         \
+    }                                                                      \
+    return 1;                                                              \
+}
+
 static int m_Object_initBool(lua_State * L)
 {
     udstruct *uin;
@@ -1033,7 +1155,7 @@ static int m_Object_initBool(lua_State * L)
     if (uin->pd != NULL && uin->pd->pc != uin->pc)
         pdfdoc_changed_error(L);
     luaL_checktype(L, 2, LUA_TBOOLEAN);
-    if (((Object *) uin->d)->getBool())
+    if (lua_toboolean(L, 2) != 0)
         ((Object *) uin->d)->initBool(gTrue);
     else
         ((Object *) uin->d)->initBool(gFalse);
@@ -1240,20 +1362,16 @@ m_poppler_get_BOOL(Object, isInt);
 m_poppler_get_BOOL(Object, isReal);
 m_poppler_get_BOOL(Object, isNum);
 m_poppler_get_BOOL(Object, isString);
-m_poppler_get_BOOL(Object, isName);
+m_Object_isType(isName);
 m_poppler_get_BOOL(Object, isNull);
 m_poppler_get_BOOL(Object, isArray);
-m_poppler_get_BOOL(Object, isDict);
-m_poppler_get_BOOL(Object, isStream);
+m_Object_isType(isDict);
+m_Object_isType(isStream);
 m_poppler_get_BOOL(Object, isRef);
-m_poppler_get_BOOL(Object, isCmd);
+m_Object_isType(isCmd);
 m_poppler_get_BOOL(Object, isError);
 m_poppler_get_BOOL(Object, isEOF);
 m_poppler_get_BOOL(Object, isNone);
-// isName
-// isDict
-// isStream
-// isCmd
 
 static int m_Object_getBool(lua_State * L)
 {
@@ -1564,6 +1682,24 @@ static int m_Object_dictAdd(lua_State * L)
     return 0;
 }
 
+static int m_Object_dictSet(lua_State * L)
+{
+    const char *s;
+    udstruct *uin, *uobj;
+    uin = (udstruct *) luaL_checkudata(L, 1, M_Object);
+    s = luaL_checkstring(L, 2);
+    uobj = (udstruct *) luaL_checkudata(L, 3, M_Object);
+    if (uin->pd != NULL && uobj->pd != NULL && uin->pd != uobj->pd)
+        pdfdoc_differs_error(L);
+    if ((uin->pd != NULL && uin->pd->pc != uin->pc)
+        || (uobj->pd != NULL && uobj->pd->pc != uobj->pd->pc))
+        pdfdoc_changed_error(L);
+    if (!((Object *) uin->d)->isDict())
+        luaL_error(L, "Object is not a Dict");
+    ((Object *) uin->d)->dictSet((char *) s, (Object *) uobj->d);
+    return 0;
+}
+
 static int m_Object_dictLookup(lua_State * L)
 {
     const char *s;
@@ -1842,6 +1978,7 @@ static const struct luaL_Reg Object_m[] = {
     {"arrayGetNF", m_Object_arrayGetNF},
     {"dictGetLength", m_Object_dictGetLength},
     {"dictAdd", m_Object_dictAdd},
+    {"dictSet", m_Object_dictSet},
     {"dictLookup", m_Object_dictLookup},
     {"dictLookupNF", m_Object_dictLookupNF},
     {"dictGetKey", m_Object_dictGetKey},
@@ -2550,6 +2687,7 @@ int luaopen_epdf(lua_State * L)
     register_meta(Annots);
     register_meta(Array);
     register_meta(Catalog);
+    register_meta(EmbFile);
     register_meta(Dict);
     register_meta(GooString);
     register_meta(LinkDest);
